/**********************************************
 ICE Proprietary Software - do NOT disseminate
 **********************************************/
/*
  Xilinx RocketIO 3GIO Controller

  Jeff Schoen
  Innovative Computer Engineering, Inc.
  1/15/2003


*/
`ifdef RIO_QPM
`define LCHN2
`else
`define LCHN1
`endif

module riocontroller (
  bsclk,bscs,bsrd,bswr,bspage,bsaddr,bsdata,bsdatao, // Boundary Scan Interface
  sclk,srst, scs,saddr, swr,swrbus, srd,srdbus,
  pxclkx, pxrxp,pxrxn, pxtxp,pxtxn, pxctl,
  rclk, clk1,rst1,rbus1,tbus1, clk2,rst2,rbus2,tbus2,
  test);			

  parameter PORTS=8;
  parameter IOTYPE1=0;
  parameter IOTYPE2=0;
  parameter IBW=64;

  localparam MPORTS=16;
  localparam DBW=64;
  localparam DBWP=4+1+1+DBW;	// hold+rdy+valid+bus

  // BSCAN ports
  input bsclk,bscs,bsrd,bswr;
  input [7:0] bspage,bsaddr,bsdata;
  output [7:0] bsdatao;

  // system bus
  input sclk,srst,scs,swr,srd;
  input [31:0] saddr,swrbus;
  output [31:0] srdbus;
        
  // MGT ports
  input [3:0] pxclkx;
  input [PORTS-1:0] pxrxp,pxrxn;
  output [PORTS-1:0] pxtxp,pxtxn;
  inout [MPORTS-1:0] pxctl;

  // IO port busses
  input rclk;
  output clk1,clk2;
  input  rst1,rst2;
  output reg [DBWP-1:0] rbus1,rbus2;
  input  [DBWP-1:0] tbus1,tbus2;

  // test port
  output [7:0] test;

  // global signals 
  wire F=0, T=1;

  wire pxclk  = pxclkx[0];
  wire pxclkp = pxclkx[2];
  wire pxclkn = pxclkx[3];

  // boundary scan monitor
  reg         drp_en,drp_en0,drp_enx;
  reg         drp_we,drp_we0,drp_wex;
  reg  [12:0] drp_addr;
  reg  [15:0] drp_di,drp_dor;
  wire        drp_rdy,drp_rdyxi;
  wire [15:0] drp_do,drp_doxi;
  wire [15:0] dmon_do,dmon_doxi;
  wire [7:0]  drp_rdyx;
  wire [PORTS*16-1:0] drp_dox;
  wire [PORTS*17-1:0] dmon_dox;
  wire        drp_clk=sclk;

  wire bsrw  = bscs && (bswr || bsrd);
  reg drpen,drpwe,drpad,drpx;
  reg [13:0] drpaddr;
  always @(posedge bsclk) begin
    drpx <= bspage[6];
    if (bsrw) drpaddr <= {bspage,bsaddr[7:1]};
    if (bsrw) drp_di <= {bsdata,drp_di[15:8]};
    if (bsrw) drpad  <= bsaddr[0];
    if (bsrw) drpwe  <= (bswr && bsaddr[0]);
    if (!bscs) drpen <= 0; else drpen <= (bswr && bsaddr[0]) || (bsrd && !bsaddr[0]);
  end
  assign bsdatao = drpad? drp_dor[15:8] : drp_dor[7:0];
  reg drpgo_,drpgo;
  always @(posedge drp_clk) begin
    drpgo   <= drpen;
    drpgo_  <= drpgo;
    drp_en  <= drpgo && !drpgo_;
    drp_we  <= drpgo && !drpgo_ && drpwe;
    drp_addr<= drpaddr;
    drp_en0 <= drp_en && !drp_addr[12];
    drp_enx <= drp_en &&  drp_addr[12];
    drp_we0 <= drp_en && drp_we && !drp_addr[12];
    drp_wex <= drp_en && drp_we &&  drp_addr[12];
    if (drpx) drp_dor <= dmon_doxi;
    else if (drp_en) drp_dor <= 16'hDEAD;
    else if (drp_rdy) drp_dor <= drp_do;
    else if (drp_rdyxi) drp_dor <= drp_doxi;
  end
  muxMxN #(PORTS,1) drpmr (drp_rdyxi, drp_rdyx, drp_addr[11:9]);
  muxMxN #(PORTS,16) drpmd (drp_doxi, drp_dox, drp_addr[11:9]);
  muxMxN #(PORTS,17) dmonmd (dmon_doxi, dmon_dox, drp_addr[11:9]);

  wire clk1 = rclk;
  wire clk2 = rclk;
  reg rst1x; always @(posedge clk1) rst1x <= rst1 || srst;
  reg rst2x; always @(posedge clk2) rst2x <= rst2 || srst;
  wire [3:0] rdy1,rdy2;
  wire [7:0] test1,test2;

  reg  [31:0] icetsys0,icetsys1;
  reg  [7:0]  icet_route;
  reg  [31:0] dbgmemaddr;
  wire [31:0] dbgmemdat;
  wire [31:0] ch0_tpkt,ch0_npkt,ch0_nerr;
  wire [31:0] ch1_tpkt,ch1_npkt,ch1_nerr;
  wire [31:0] ch2_tpkt,ch2_npkt,ch2_nerr;
  wire [31:0] ch3_tpkt,ch3_npkt,ch3_nerr;
  wire [31:0] ch4_tpkt,ch4_npkt,ch4_nerr;
  wire [31:0] ch5_tpkt,ch5_npkt,ch5_nerr;
  wire [31:0] ch6_tpkt,ch6_npkt,ch6_nerr;
  wire [31:0] ch7_tpkt,ch7_npkt,ch7_nerr;
  wire [63:0] ch0_lcorr,ch1_lcorr,ch2_lcorr,ch3_lcorr;
  wire [63:0] ch7_lcorr,ch6_lcorr,ch5_lcorr,ch4_lcorr;
  wire [31:0] txaux1,txaux2,rxaux1,rxaux2;
  wire [31:0] dbgsys = 32'h12345678;
  assign txaux1 = tbus1[DBWP-1:DBWP-4];
  assign txaux2 = tbus2[DBWP-1:DBWP-4];

  // system register address resolution - with pipeline
  wire [15:0] sb; swrbank #(0,16) swb (sclk,scs,swr,saddr,sb);
  wire swrsys0  = sb[0];
  wire swrsys1  = sb[1];
  wire swrdbgm  = sb[2];
  wire swrirte  = sb[3];
  
  // system registers - written to by pic_cport() host control software
  always @(posedge sclk) begin
    if (srst) icetsys0   <= 0; else if (swrsys0) icetsys0   <= swrbus;
    if (srst) icetsys1   <= 0; else if (swrsys1) icetsys1   <= swrbus;
    if (srst) icet_route <= 0; else if (swrirte) icet_route <= swrbus[7:0];
    if (srst) dbgmemaddr <= 0; else if (swrdbgm) dbgmemaddr <= swrbus;
  end

  // this mux concatenates all inputs from right-to-left in second argument
  muxMxNp #(32,32) (srdbus,{ch7_tpkt,ch6_tpkt,ch5_tpkt,ch4_tpkt,ch3_tpkt,ch2_tpkt,ch1_tpkt,ch0_tpkt,
			    ch7_nerr,ch6_nerr,ch5_nerr,ch4_nerr,ch3_nerr,ch2_nerr,ch1_nerr,ch0_nerr,
			    ch7_npkt,ch6_npkt,ch5_npkt,ch4_npkt,ch3_npkt,ch2_npkt,ch1_npkt,ch0_npkt,
	ch1_lcorr[63:0],dbgsys,24'hDEAD00,icet_route, dbgmemdat,dbgmemaddr,icetsys1,icetsys0}, saddr[6:2],sclk);

`ifdef LCHN4

  wire [DBW:0] rbus11,rbus12,rbus13,rbus14;
  wire [DBW:0] rbus21,rbus22,rbus23,rbus24;
  icet_lcl #(4,DBW) bnk1 (clk1,rst1x,rdy1, {4{tbus1[DBW]}},{4{tbus1[DBW-1:0]}}, 
                {rbus14[DBW],rbus13[DBW],rbus12[DBW],rbus11[DBW]},{rbus14[DBW-1:0],rbus13[DBW-1:0],rbus12[DBW-1:0],rbus11[DBW-1:0]}, txaux1,rxaux1,
		drp_clk,F,drp_enx,drp_addr,drp_wex,drp_di,drp_rdyx[3:0],drp_dox[16*4-1:16*0],dmon_dox[17*4-1:17*0],
		pxclk, pxtxp[3:0],pxtxn[3:0], pxrxp[3:0],pxrxn[3:0], test1,icetsys0, 
                {ch3_npkt,ch2_npkt,ch1_npkt,ch0_npkt},{ch3_nerr,ch2_nerr,ch1_nerr,ch0_nerr},{ch3_tpkt,ch2_tpkt,ch1_tpkt,ch0_tpkt});
`ifndef ICE_PM
  icet_lcl #(4,DBW) bnk2 (clk2,rst2x,rdy2, {4{tbus2[DBW]}},{4{tbus2[DBW-1:0]}},
                {rbus24[DBW],rbus23[DBW],rbus22[DBW],rbus21[DBW]},{rbus24[DBW-1:0],rbus23[DBW-1:0],rbus22[DBW-1:0],rbus21[DBW-1:0]}, txaux2,rxaux2,
		drp_clk,F,drp_enx,drp_addr,drp_wex,drp_di,drp_rdyx[7:4],drp_dox[16*8-1:16*4],dmon_dox[17*8-1:17*4],
		pxclk, pxtxp[7:4],pxtxn[7:4], pxrxp[7:4],pxrxn[7:4], test2,icetsys1,
                {ch7_npkt,ch6_npkt,ch5_npkt,ch4_npkt},{ch7_nerr,ch6_nerr,ch5_nerr,ch4_nerr},{ch7_tpkt,ch6_tpkt,ch5_tpkt,ch4_tpkt});
`endif
  always @(posedge clk1) rbus1 <= {rxaux1[3:0],rdy1[0],rbus11[DBW:0]};
  always @(posedge clk2) rbus2 <= {rxaux2[3:0],rdy1[1],rbus12[DBW:0]};

`elsif LCHN2

  wire [DBW:0] rbus11,rbus12,rbus21,rbus22;
  icet_lcl #(2,DBW) bnk1 (clk1,rst1x,rdy1, {tbus2[DBW],tbus1[DBW]},{tbus2[DBW-1:0],tbus1[DBW-1:0]}, {rbus12[DBW],rbus11[DBW]},{rbus12[DBW-1:0],rbus11[DBW-1:0]}, txaux1,rxaux1,
		drp_clk,F,drp_enx,drp_addr,drp_wex,drp_di,drp_rdyx[3:0],drp_dox[16*4-1:16*0],dmon_dox[17*4-1:17*0],
		pxclk, pxtxp[3:0],pxtxn[3:0], pxrxp[3:0],pxrxn[3:0], test1,icetsys0, 
                {ch3_npkt,ch2_npkt,ch1_npkt,ch0_npkt},{ch3_nerr,ch2_nerr,ch1_nerr,ch0_nerr},{ch3_tpkt,ch2_tpkt,ch1_tpkt,ch0_tpkt});
`ifndef ICE_PM
  icet_lcl #(2,DBW) bnk2 (clk2,rst2x,rdy2, {tbus1[DBW],tbus2[DBW]},{tbus1[DBW-1:0],tbus2[DBW-1:0]}, {rbus22[DBW],rbus21[DBW]},{rbus22[DBW-1:0],rbus21[DBW-1:0]}, txaux2,rxaux2,
		drp_clk,F,drp_enx,drp_addr,drp_wex,drp_di,drp_rdyx[7:4],drp_dox[16*8-1:16*4],dmon_dox[17*8-1:17*4],
		pxclk, pxtxp[7:4],pxtxn[7:4], pxrxp[7:4],pxrxn[7:4], test2,icetsys1,
                {ch7_npkt,ch6_npkt,ch5_npkt,ch4_npkt},{ch7_nerr,ch6_nerr,ch5_nerr,ch4_nerr},{ch7_tpkt,ch6_tpkt,ch5_tpkt,ch4_tpkt});
`endif
  always @(posedge clk1) rbus1 <= {rxaux1[3:0],rdy1[0],rbus11[DBW:0]};
  always @(posedge clk2) rbus2 <= {rxaux2[3:0],rdy1[1],rbus12[DBW:0]};

`elsif LCHN1

  wire [DBW:0] rbus11,rbus12,rbus21,rbus22;
  icet_lcl #(1,DBW) bnk1 (clk1,rst1x,rdy1, tbus1[DBW],tbus1[DBW-1:0], rbus11[DBW],rbus11[DBW-1:0], txaux1,rxaux1,
		drp_clk,F,drp_enx,drp_addr,drp_wex,drp_di,drp_rdyx[3:0],drp_dox[16*4-1:16*0],dmon_dox[17*4-1:17*0],
		pxclk, pxtxp[3:0],pxtxn[3:0], pxrxp[3:0],pxrxn[3:0], test1,icetsys0, 
                {ch3_npkt,ch2_npkt,ch1_npkt,ch0_npkt},{ch3_nerr,ch2_nerr,ch1_nerr,ch0_nerr},{ch3_tpkt,ch2_tpkt,ch1_tpkt,ch0_tpkt});
`ifndef ICE_PM
  icet_lcl #(1,DBW) bnk2 (clk2,rst2x,rdy2, tbus2[DBW],tbus2[DBW-1:0], rbus21[DBW],rbus21[DBW-1:0], txaux2,rxaux2,
		drp_clk,F,drp_enx,drp_addr,drp_wex,drp_di,drp_rdyx[7:4],drp_dox[16*8-1:16*4],dmon_dox[17*8-1:17*4],
		pxclk, pxtxp[7:4],pxtxn[7:4], pxrxp[7:4],pxrxn[7:4], test2,icetsys1,
                {ch7_npkt,ch6_npkt,ch5_npkt,ch4_npkt},{ch7_nerr,ch6_nerr,ch5_nerr,ch4_nerr},{ch7_tpkt,ch6_tpkt,ch5_tpkt,ch4_tpkt});
`endif
  always @(posedge clk1) rbus1 <= {rxaux1[3:0],rdy1[0],rbus11[DBW:0]};
  always @(posedge clk2) rbus2 <= {rxaux2[3:0],rdy2[0],rbus21[DBW:0]};

`endif
//  assign test = {rdy2,rdy1,clk2,clk1,tbus2[32],tbus1[32],rbus2[32],rbus1[32]};
//  assign test = {test1,tbus1[DBW],rbus1[DBW],clk1,srst};
  assign test = test1;

endmodule

