/**********************************************
 ICE Proprietary Software - do NOT disseminate
 **********************************************/
/*
  ICEPIC-8+ Ethernet Module Controller

  Jeff Schoen
  Innovative Computer Engineering, Inc.
  12/15/2004

  Packet Contents

*/
`include "vdefs.h"
`include "../lib/ramdefs.h"
`include "../lib/hypdefs.h"

module emodule (
  sclk,rst, scs,saddr, swr,swrbus, srd,srdbus, sack,
  rclk,rrst,ribus,robus, 
`ifdef JEFF
  ioclk, istat,iena,ibus,isel, ostat,oena,obus,osel,
  istat1,istat2,ostat1,ostat2,mcena,
`endif
  test);			

  parameter PORT=1;
  parameter IBW=64;
  parameter MODE=5;

  localparam DBW=64;
  localparam DBV=DBW/8;
  localparam IBV=IBW/8;
  localparam DBWP=DBW+1;

  // system bus
  input sclk,rst,scs,swr,srd;
  input [31:0] saddr,swrbus;
  output [31:0] srdbus;
  output sack;

  // RIO Interface
  input rclk,rrst;
  input [DBWP:0] ribus;
  output [DBWP:0] robus;

  // DMA bus
`ifdef JEFF
  input ioclk,iena,oena;
  input [IBW-1:0] ibus;
  input [7:0] isel;
  output [IBW-1:0] obus;
  output [7:0] osel;
  output istat,ostat;

  // direct
  output istat1,istat2;
  output ostat1,ostat2;
  input [7:0] mcena;
`else
  wire [7:0] mcena, isel,osel;
  wire oena,iena;
`endif

  // test port
  output [7:0] test;

  // global signals
  wire L=0, H=1;
  reg [31:0] pm;
  assign sack=H;

  wire good = H;
  wire ena  = pm[0];
  wire orst = pm[16];   // test reset
  wire irst = pm[17];   // test reset
  wire mode = pm[31];   // raw data collect

  reg enb,enc,dclkrst,dclkadj,dclkud;
  wire swriom = (scs && swr && saddr[5:2]==0);
  wire swriop = (scs && swr && saddr[5:0]==6);
  always @(posedge sclk) begin
    if (swriom) pm <= swrbus;
    dclkrst <= swriop && swrbus[0];
    dclkadj <= swriop && swrbus[1];
    dclkud  <= swriop && swrbus[8];
    enb     <= ena && !rst;
    enc     <= enb && !mode;
  end
  wire frst = !enb;
  wire [3:0] vadj = {dclkud,dclkadj,dclkrst,sclk};

  // channel wires
  wire wvalix,wvalox;
  wire [1:0] wrdyo;
  wire [2:0] wrdyi;
  wire [DBV-1:0] wchni,wchno;
  wire [DBW-1:0] wbusi,wbuso;
  reg  [2:0] icnt,ocnt;
  reg ienad;
  always @(posedge ioclk) begin
    if (!oena) ocnt <= 0; else ocnt <= ocnt+1;
    if (iena&!ienad) icnt <= 1; else icnt <= icnt+1;
    ienad <= iena;
  end
  wire ichi  = isel[0];
  wire iten  = isel[1];
  wire isen  = isel[2];
  wire itena = isel[4];
  wire isena = isel[5];
  wire itenb = isel[6];
  wire isenb = isel[7];
  wire [IBV-1:0] ichn = {2{iten,iten,isen,ichi,isen,ichi,isen,ichi}};
  wire [7:0] votest,vitest,calib;
  wire [3:0] ihold,ohold;

   wire vclk   = rclk;
   wire fclk   = rclk;
   wire vrdy   = ribus[DBWP];

  // multichan aggregators
  reg mcd; always @(posedge ioclk) mcd <= mcena[3];
  wire csm,cab; // channel single/multi and A/B 
  wire [2:0] wvali = !wvalix? 3'b0 : (mcd&&~csm)? {cab,~cab,L} : {L,L,H}; 
  assign ihold = !enc? 0 : mcd? {L,~wrdyi[2:0]} : {L,{3{~wrdyi[0]}}};
  wire [2:0] oenas = mcena[2:0];

  // output dual aggregators
  reg [1:0] ivals; always @(posedge ioclk) ivals <= mcena[5:4]; 
  reg [1:0] och; always @(posedge vclk) och <= (wrdyo[1] && !ohold[2])? 2 : (wrdyo[0] && !ohold[1])? 1 : 0;
  reg [1:0] ocho; always @(posedge vclk) if (!wvalox && och!=0) ocho <= och;
  reg vrdyd;    always @(posedge vclk) vrdyd <= vrdy;
  wire [1:0] wvalo = !wvalox? 0 : ocho;
  wire wrdyx = (och!=0) && vrdyd;

  // the qdr output packet handler
  assign istat = istat1 && istat2;
  dcfifoNxM #(IBW+IBV,DBW+DBV,2,3,`BPAR,`PACKET) fo (ioclk,frst,{istat2,istat1},ivals,{ichn,ibus}, vclk,frst,wrdyo,wvalo,{wchno,wbuso});
  hypoutr #(DBW,DBW,`DMA_PKT_LEN,`HYP_ONOF) qhypo (vclk,enb, robus[DBW],robus[DBW-1:0], wrdyx,wvalox,wchno,wbuso, ihold,votest);

  // the qdr input packet handler
  wire [31:0] vdati = 0;	// for ctest
  hypinr #(DBW,DBW,`DMA_PKT_LEN,`HYP_ONOF) qhypi (fclk,enb, ribus[DBW],ribus[DBW-1:0], wvalix,wchni,wbusi, mode,ohold,csm,cab,calib,vitest);
  assign rtest = {wvalix,wchni[7:0],wbusi[23:0]};
  mcfifoNxM #(DBW+DBV,IBW+IBV,4,4,`BPAR,`PACKET) fi (fclk,frst, wrdyi,wvali,{wchni,wbusi}, ioclk,frst, {ostat2,ostat1,ostat},oenas,{osel,obus});

  assign test = 0;
//  assign test = {ohold,wvalox,wvalix,oena,iena};

endmodule

