/*
  Verilog modules for simulating various Lime primitives

  Jeff Schoen
  Innovative Computer Engineering, Inc.
  11/15/2013

*/
module lime();

endmodule

module sync_fifo (clk,reset, wr_req,din,full, rd_req,dout,empty, space_remaining);

  parameter DATA_WIDTH=128;
  parameter FIFO_DEPTH=16;
  parameter IMP_TYPE="distributed";

  input clk,reset;
  input wr_req,rd_req;
  input [DATA_WIDTH-1:0] din;
  output [DATA_WIDTH-1:0] dout;
  output full,empty;
  output [7:0] space_remaining;

  wire stat,afull;
  que16xNaf qq (clk,reset, wr_req,rd_req, din,dout, stat,afull);
  assign empty = !stat;
  assign full = afull;
  assign space_remaining = afull? 0 : stat? 8 : 15; 

endmodule


module onchipRAM (reset, clk, enable1, write1, addr1, dataIn1, dataOut1, 
                              enable2, write2, addr2, dataIn2, dataOut2, mem_init_req, init_value, init_busy); 

  parameter MEM_DEPTH=128;
  parameter DATA_WIDTH=16;
  parameter IMP_TYPE="block";
  parameter INIT_MEM_CONST="false";
  parameter INIT_MEM_CONST_VALUE=0;

  localparam ABW=16;
  localparam DBW=DATA_WIDTH;

  input reset;
  input clk;

  input enable1;
  input write1;
  input [ABW-1:0] addr1;
  input [DBW-1:0] dataIn1;
  output [DBW-1:0] dataOut1;

  input enable2;
  input write2;
  input [ABW-1:0] addr2;
  input [DBW-1:0] dataIn2;
  output [DBW-1:0] dataOut2;

  input mem_init_req;
  input init_value;
  output init_busy;


endmodule
