/**********************************************
 ICE Proprietary Software - do NOT disseminate
 **********************************************/
/*
  PowerPC Processor Interface

  Jeff Schoen
  Innovative Computer Engineering, Inc.
  1/15/2003

  iena/oena are not one cycle early in this module

  iosize: 0=4by 1=16by 2=32by 3=64by

*/
`include "vdefs.h"
`include "mdefs.h"

`ifndef JVM_NR
`define JVM_NR 4
`endif

`ifndef JVM_NCR
`define JVM_NCR 0
`endif

`define PRC_SBUS 1
`define PRC_FBUS 1

module processor (
  cclk,crd,cwr,cpage,caddr,cdata,cdatao, cs,		// JTAG IF
  bclk,baddr, brd,brdbus,bwr,bwrbus, boot,     		// PPC BRam IF
  sclkx,srst, srw,sadx, srdx,srdbus, swrx,swrbus, sack,// System Control
`ifdef PRC_SBUS
  sclks,srds,swrs,sads,					// Altername SC rates
`endif
`ifdef PRC_FBUS
  sclkfs,srdfs,swrfs,sadf,					// Altername SC rates
`endif
`ifdef PRC_TBUS
  sclkt,srdt,swrt,sadt,					// Altername SC rates
`endif
  ioclk, ireq,iena,ibus, oreq,oena,obus, 		// IO busses
  ioaddr,iosize,iobusy,iok,ioreq,			// IO signals
  test);

  parameter IBW=32;
  parameter MODE=1;	// mode 0=mainboard 1=processor
  parameter DSMODE=1;	// mode 1=fullSpeedMem 2=halfSpeedMem
  parameter ISMODE=1;	// mode 1=fullSpeedMem 2=halfSpeedMem

  localparam DSBW=32;
  localparam ISBW=64;
  localparam NR=`JVM_NR;
  localparam NCR=`JVM_NCR;

  input cclk,crd,cwr,cs;
  input [7:0] cpage,caddr,cdata;
  output [7:0] cdatao;

  input bclk,boot;
  output brd,bwr;
  output [15:0] baddr;
  input [31:0] brdbus;
  output [31:0] bwrbus;

  input srst,sclkx;
  input [31:0] srdbus;
  output srw,srdx,swrx;
  output [31:0] swrbus,sadx;
`ifdef PRC_SBUS
  input sclks;
  output srds,swrs;
  output reg [31:0] sads;
`endif
`ifdef PRC_FBUS
  input [1:0] sclkfs;
  output [1:0] srdfs,swrfs;
  output reg [31:0] sadf;
`endif
`ifdef PRC_TBUS
  input sclkt;
  output srdt,swrt;
  output reg [31:0] sadt;
`endif
  input [7:0] sack;

  input ioclk,iena,oena,iobusy;
  output ireq,oreq,iok;
  output [31:0] ioaddr;
  output [1:0] iosize;
  output [31:0] ioreq;

  output [IBW-1:0] obus;
  input [IBW-1:0] ibus;

  output [7:0] test;

  // Power assignments
  wire H=1, L=0;

`ifdef PRC_BCLK
  wire sclky = bclk;
  wire dsclk = bclk;
  wire isclk = bclk;
`else
  wire sclky = sclkx;
  wire dsclk = (DSMODE<=1)? sclkx : bclk;
  wire isclk = (ISMODE<=1)? sclkx : bclk;
`endif

  // Debug, status and reset logic
  reg rst,iok,creset,cstart;
  always @(posedge sclky) begin
`ifdef JVM
    rst <= srst || creset;
`else
    rst <= srst;
`endif
    iok <= !rst;
  end

  // OCM - OnChip Memory Controllers 
  wire isen,dsen,dwen;
  wire [20:0] isadr;
  wire [21:0] dsadr;
  wire [ISBW-1:0] isbus;


  // Control port interface
  reg [15:0] ccnt;
  reg [31:0] cdataw;
  reg iswr,dswr,cinc;
  reg iswrc,iswrd;
  reg ciswr,ciswrd;
  wire [7:0] cdatad = cdataw[31:24];
  wire cadr = cs && cwr && caddr[2];
  wire csys = cs && cwr && caddr[3];
  wire dswr_ = cs && cwr && caddr[3:2]==0 && !ccnt[14] && !ccnt[13];
  wire iswr_ = cs && cwr && caddr[3:2]==0 && !ccnt[14] &&  ccnt[13];
  always @(posedge cclk) begin
    if (cadr) ccnt <= {cdata,cdatad};
    else if (cinc) ccnt <= ccnt+1;
    if (cwr) cdataw <= {cdata,cdataw[31:8]};
    cinc <= cs && ((cwr && caddr[3:2]==0) || crd);
    // need to change how we do this for more data space
    dswr <= dswr_;
    iswr <= iswr_;
  end
  always @(posedge cclk or posedge srst) begin
    if (srst) cstart <= 0; else if (csys) cstart <= cdata[0];
    if (srst) creset <= 0; else if (csys) creset <= cdata[1];
  end

  // system bus
  wire [31:0] saddr;
  wire swr_en,srd_en;

`ifdef PRC_BCLK
  reg [31:0] sadx;
  reg swrx,srdx,swrx_,srdx_,srwx;
  always @(posedge sclky or posedge srwx) begin
    if (srwx) srdx_ <= 0; else srdx_ <= srd_en || srdx_;
    if (srwx) swrx_ <= 0; else swrx_ <= swr_en || swrx_;
  end
  always @(posedge sclkx) begin
    srdx <= srdx_ && !srdx;
    swrx <= swrx_ && !swrx;
    sadx <= saddr;
  end
  always @(negedge sclkx) srwx <= srdx || swrx || srst;

`else
  reg srdx,swrx;
  always @(posedge sclkx) begin
    if (srst) srdx <= 0; else srdx <= srd_en;
    if (srst) swrx <= 0; else swrx <= swr_en;
  end
  assign sadx = saddr;
`endif

`ifdef PRC_SBUS
  reg swrs,srds,swrs_,srds_,srws;
  always @(posedge sclky or posedge srws) begin
    if (srws) srds_ <= 0; else srds_ <= srd_en || srds_;
    if (srws) swrs_ <= 0; else swrs_ <= swr_en || swrs_;
  end
  always @(posedge sclks) begin
    srds <= srds_ && !srds;
    swrs <= swrs_ && !swrs;
    sads <= saddr;
  end
  always @(negedge sclks) srws <= srds || swrs || srst;
`endif

`ifdef PRC_FBUS
  wire sclkf = sclkfs[0];
  reg swrf,srdf,swrf_,srdf_,srwf;
  always @(posedge sclky or posedge srwf) begin
    if (srwf) srdf_ <= 0; else srdf_ <= srd_en || srdf_;
    if (srwf) swrf_ <= 0; else swrf_ <= swr_en || swrf_;
  end
  always @(posedge sclkf) begin
    srdf <= srdf_ && !srdf;
    swrf <= swrf_ && !swrf;
    sadf <= saddr;
  end
  always @(negedge sclkf) srwf <= srdf || swrf || srst;
  assign swrfs[0] = swrf;
  assign srdfs[0] = srdf;
`ifdef SYS_CLKF2
  wire sclkf2 = sclkfs[1];
  reg swrf2; always @(posedge sclkf2) swrf2 <= swrf && !swrf2;
  reg srdf2; always @(posedge sclkf2) srdf2 <= srdf && !srdf2;
  assign swrfs[1] = swrf2;
  assign srdfs[1] = srdf2;
`endif
`endif

`ifdef PRC_TBUS
  reg swrt,srdt,swrt_,srdt_,srwt;
  always @(posedge sclky or posedge srwt) begin
    if (srwt) srdt_ <= 0; else srdt_ <= srd_en || srdt_;
    if (srwt) swrt_ <= 0; else swrt_ <= swr_en || swrt_;
  end
  always @(posedge sclkt) begin
    srdt <= srdt_ && !srdt;
    swrt <= swrt_ && !swrt;
    sadt <= saddr;
  end
  always @(negedge sclkt) srwt <= srdt || swrt || srst;
`endif


`ifdef JVM

  wire [2:0] dswen;
  wire [DSBW-1:0] dsbusi,dsbuso;
  assign bwrbus = dsbuso[31:0];
  // these ccnt addresses are good to 64Kby and conflict with ISWR above 8Kby
  dpram #(NR,8,DSBW) dsr (cclk,H, ccnt,dswr, cdatad,cdatao, 
                          dsclk,H, dsadr,dswen[0], dsbuso,dsbusi);
  assign bwr   = dswen[2];
  assign dsadr = baddr[15:2];

  reg [3:0] cunb;
  always @(posedge cclk) begin
    if (srst) cunb  <= 0; else if (cwr) cunb <= caddr[3:0];
    if (srst) iswrc <= 0; else if (iswr) iswrc <= !iswrc;
  end

  reg clc,cl1,cl2;
  reg [11:0] code,csum;
  always @(posedge sclky) begin
    if (srst) clc <= 0; else if (cl1) clc <= !clc;
    cl1 <= (clc^iswrc) && !cl1;
    cl2 <= cl1;
    if (cl1) code <= {cunb,cdatad};
    if (srst) csum <= ~0; else if (cload) csum <= {csum[10:0],csum[11]} ^ code;
  end
  wire cload = cl2;
  wire brd_; assign brd = H; // now enables the DMAC ram
  wire [7:0] testj;

  jvm #(IBW,NCR) j1 (sclky, rst, cstart,cload,code,
                     sclky, srw, saddr,swr_en,swrbus,srd_en,srdbus,
                     dsclk, baddr,dswen,dsbuso,brd_,dsbusi,brdbus,
                     ioclk, ioaddr,iosize, ireq,iena,ibus, oreq,oena,obus, testj);

  assign test = testj;
//  assign test = csum;

`else
  
  // DSOCM input selector
  wire [3:0]  dswen;
  wire [DSBW-1:0] dsbusi,dsbuso,dsbus0;
  wire sel_ = dsadr[21];
  reg sel; always @(posedge dsclk) sel <= sel_;
`ifdef DTDM
  mux2xN #(16) dsbl (dsbusi[15:00],dsbus0[15:00],brdbus[15:00],sel);
  mux2xN #(16) dsbu (dsbusi[31:16],dsbus0[31:16],brdbus[31:16],sel);
`else
  mux2xN #(DSBW) dsb (dsbusi,dsbus0,brdbus,sel);
`endif

  dpram #(4,8,DSBW,0,`EINV) dsr (cclk,H, ccnt[11:0],dswr, cdatad,cdatao, 
                                 dsclk,sel_, dsadr[9:0],dwen, dsbuso,dsbus0); // uses ~sel as ENA

  // ISOCM Cache line handling
  always @(posedge cclk or posedge iswrc) begin
    if (iswrc) ciswr  <= 0; else ciswr  <= cs && cwr && !caddr[2] && ccnt[14] && (ccnt[1:0]==3);
    if (iswrc) ciswrd <= 0; else ciswrd <= ciswr || ciswrd;
  end
  always @(posedge sclkx) begin
    iswrd <= ciswrd;
    iswrc <= iswrd && !iswrc;
  end

  wire [7:0] cdataod;
  dpram #(4,8,ISBW) isr (cclk,H, ccnt[11:0],iswr, cdatad,cdataod, 
                         isclk,H, isadr[8:0],L, 64'h0,isbus);

  assign bwrbus = dsbuso[31:0];
  assign baddr = {dsadr[13:0],L,L};
  assign bwr = dwen;
  assign brd = sel_;

  wire [9:0] dcr_adr;
  wire [31:0] dcr_busi,dcr_buso;
  wire dcr_rd, dcr_wr, dcr_ack;

  // this stuff sits on one side of the PPC
  proc_dcr dcr (sclkx, srw, srd_en, swr_en, saddr,swrbus,
    dcr_wr, dcr_rd, dcr_adr[1:0], dcr_ack, dcr_buso);
  assign dcr_busi = srdbus;
  assign dwen  = dswen[0];

  // PLB Data/Instruction Cache Unit
  wire ibreq, dbreq, dbrnw, dbsize, busy;
  wire [1:0] ibsize;
  wire [31:0] ibadr, dbadr;
  wire ibaack, ibrack, dbaack, dbrack, dbwack;
  wire [2:0] ibcnt, dbcnt;
  wire [63:0] plb_irbus, plb_drbus, plb_dwbus;
  wire bus64;
  generate 
  if (IBW == 32) begin:b32
    assign plb_irbus = (boot)? {cdataw,cdataw} : {ibus,ibus};
    assign plb_drbus = {ibus,ibus};
    assign obus = plb_dwbus[63:32];
    assign bus64 = L;
  end
  else begin:b64
    assign plb_irbus = (boot)? {cdataw,cdataw} : ibus;
    assign plb_drbus = {ibus[31:0],ibus[63:32]};
    assign obus = {plb_dwbus[31:0],plb_dwbus[63:32]};
    assign bus64 = H;
  end
  endgenerate

  // handle early IO signals
  reg ival; always @(posedge ioclk) ival <= iena;
  reg oval; always @(posedge ioclk) oval <= oena;

  // count/address generators
  proc_plb plb (rst, 
    ioclk, ireq, oreq, ival, oval, ioaddr, iosize, iobusy, bus64,
    ibreq, ibaack, ibrack, ibsize, ibadr, ibcnt, boot, iswrc, busy,
    dbreq, dbaack, dbrack, dbsize, dbadr, dbcnt, dbrnw, dbwack
  );

  // direct access signals
  assign ioreq[31:3] = dbadr[31:3];
  assign ioreq[2] = dbsize;
  assign ioreq[1] = dbreq &&  dbrnw && !dbaack;
  assign ioreq[0] = dbreq && !dbrnw && !dbaack;

  // resets
  reg sysrst,chiprst,corerst;
  wire sysrstreq,chiprstreq,corerstreq;
  always @(posedge sclkx) begin
    sysrst  <= rst || sysrstreq;
    chiprst <= rst || chiprstreq;
    corerst <= rst || corerstreq;
  end
  
  // Constant assignments
  `define DSARC   8'hFD	 // address [31:24]
  `define DSCNTL1 8'h81	 // DMEN 1:1
  `define DSCNTL2 8'h83	 // DMEN 2:1 
  `define DSDCRA  8'h07
  `define ISARC   8'hFF	 // address [31:24]
  `define ISCNTL1 8'h81	 // IMEN 1:1
  `define ISCNTL2 8'h83	 // IMEN 2:1
  `define ISDCRA  8'h06
  `define DCRA    5'h01

  // Unisim PPC module
`ifdef VIRTEX4
  PPC405_ADV ppc1 (
`else
  PPC405 ppc1 (
`endif
    	// Clock and Power management
	.CPMC405CLOCK (sclkx),
	.CPMC405CPUCLKEN (H),
	.CPMC405JTAGCLKEN (H),
	.CPMC405TIMERCLKEN (L),
	.MCBCPUCLKEN (L),
	.MCBJTAGEN (L),
	.MCBTIMEREN (L),

	// CPU Control Interface
	.TIEC405MMUEN (L),
	.TIEC405DISOPERANDFWD (L),
	.TIEC405DETERMINISTICMULT (L),
	.C405XXXMACHINECHECK (uhoh),

	// External Interrupts
	.EICC405CRITINPUTIRQ (L),
	.EICC405EXTINPUTIRQ (L),

	// Reset Interface
	.C405RSTSYSRESETREQ (sysrstreq),
	.C405RSTCHIPRESETREQ (chiprstreq),
	.C405RSTCORERESETREQ (corerstreq),
	.RSTC405RESETSYS  (sysrst),
	.RSTC405RESETCHIP (chiprst),
	.RSTC405RESETCORE (corerst),

	// Instruction Side Processor Local Bus
	.C405PLBICUABUS (ibadr[31:2]),
	.C405PLBICUREQUEST (ibreq),
	.C405PLBICUSIZE (ibsize),  		// 01=16by, 10=32by
	.PLBC405ICUADDRACK (ibaack),
	.PLBC405ICUBUSY (busy),
	.PLBC405ICURDDACK (ibrack),
	.PLBC405ICURDDBUS (plb_irbus),
	.PLBC405ICURDWDADDR (ibcnt),
	.PLBC405ICUSSIZE1 (!boot), 		// L=32b, H=64b

	// Data Side Processor Local Bus
	.C405PLBDCUABUS (dbadr),
	.C405PLBDCURNW (dbrnw),
	.C405PLBDCUREQUEST (dbreq),
	.C405PLBDCUWRDBUS (plb_dwbus),
	.C405PLBDCUSIZE2 (dbsize),  		// 0=4by, 1=32by
	.PLBC405DCUADDRACK (dbaack),
	.PLBC405DCUBUSY (busy),
	.PLBC405DCURDDACK (dbrack),
	.PLBC405DCURDDBUS (plb_drbus),
	.PLBC405DCURDWDADDR (dbcnt),
	.PLBC405DCUWRDACK (dbwack),
	.PLBC405DCUSSIZE1 (bus64), 		// L=32b, H=64b

	// DSOCM Interface
	.DSARCVALUE (`DSARC),
	.DSCNTLVALUE ((DSMODE==2)?`DSCNTL2:`DSCNTL1),
	.BRAMDSOCMCLK (dsclk),
	.BRAMDSOCMRDDBUS (dsbusi),
	.DSOCMBRAMABUS (dsadr),
	.DSOCMBRAMEN (dsen),
	.DSOCMBRAMBYTEWRITE (dswen),
	.DSOCMBRAMWRDBUS (dsbuso),
`ifdef VIRTEX4
	.DSOCMRWCOMPLETE (H),
`endif

	// ISOCM Interface
	.ISARCVALUE (`ISARC),
	.ISCNTLVALUE ((ISMODE==2)?`ISCNTL2:`ISCNTL1),
	.BRAMISOCMCLK (isclk),
	.BRAMISOCMRDDBUS ({isbus[31:0],isbus[63:32]}),
	.ISOCMBRAMEN (isen),
	.ISOCMBRAMRDABUS (isadr),

	// DCR Interface
`ifdef VIRTEX4
	.EXTDCRABUS (dcr_adr),
	.EXTDCRDBUSOUT (dcr_buso),
	.EXTDCRREAD (dcr_rd),
	.EXTDCRWRITE (dcr_wr),
	.EXTDCRACK (dcr_ack),
	.TIEDCRADDR (`DCRA),
	.EXTDCRDBUSIN (dcr_busi),
`else
	.C405DCRABUS (dcr_adr),
	.C405DCRDBUSOUT (dcr_buso),
	.C405DCRREAD (dcr_rd),
	.C405DCRWRITE (dcr_wr),
	.DCRC405ACK (dcr_ack),
	.TIEISOCMDCRADDR (`ISDCRA),
	.TIEDSOCMDCRADDR (`DSDCRA),
	.DCRC405DBUSIN (dcr_busi),
`endif
	// Additional FPGA Signals
	.PLBCLK (sclkx)
	);
  assign test = {dcr_rd, dcr_wr, ibreq, dbreq, isen, dsen, sclkx, rst};
`endif

//  assign test = {iok,dbwack,dbrack,dbaack,dbreq,busy,ibaack,ibreq};
//  assign test = {dcr_buso[1:0],dcr_adr[1:0], dcr_wr,dcr_rd, swr_en,rst};
//  assign test = {dcr_buso[1:0],swrbus[1:0], dcr_wr,dcr_rd, swr_en,rst};
//  assign test = {iobusy, busy,  dbreq, dbaack, dbrack, dbsize, dbrnw, dbwack};
//  assign test = 0;

endmodule

/*

  PLB Timing

	dbreq
	dbreq	dbaack
bio			iordy
bio	dbreq		iordy	oena
bio	dbreq		irdy	oena
bi	dbreq	dbaack	irdy	ioena
	dbreq	dbaack		ioena
	dbreq		odone	iena
	dbreq	dbaack		iena
	dbreq		idone
	dbreq


*/

module proc_plb (rst, 
    ioclk, ireq, oreq, iena, oena, ioaddr, iosize, iobusy, bus64,
    ibreq, ibaack, ibrack, ibsize, ibadr, ibcnt, boot, iswr, busy,
    dbreq, dbaack, dbrack, dbsize, dbadr, dbcnt, dbrnw, dbwack
  );

  input rst, ioclk, iena, oena, boot, iswr, iobusy, ibreq, dbreq, dbrnw, dbsize, bus64;
  input [31:0] ibadr, dbadr;
  input [1:0] ibsize;
  output ireq, oreq, ibaack, ibrack, dbaack, dbrack, dbwack, busy;
  output [31:0] ioaddr;
  output [2:0] ibcnt, dbcnt;
  output [1:0] iosize;

  wire H=1, L=0;
  reg ireq, oreq, ioreq, ibaack, dbaack;
  reg iside, hold, busy, ioenad, bdone;
  reg [1:0] iosize;
  reg [31:0] ioaddr;
  reg [2:0] ibcnt,dbcnt;

  assign ibrack = (iena &&  iside) || iswr;
  assign dbrack = (iena && !iside);
  assign dbwack = oena;

  wire ioena = iena || oena;
  wire idbaack = ibaack || dbaack;
  wire done = (ioenad && !(iena || oena)) || bdone;

  wire [1:0] ibinc = (bus64&!boot)? 2:1;
  wire [1:0] dbinc = (bus64)? 2:1;

  always @(posedge ioclk) begin
 
    // prefer data over instruction 
    ibaack <= ibreq && !ibaack && !hold && !iobusy; // && !dbreq;
    dbaack <= dbreq && !dbaack && !hold && !iobusy && !ibreq;

    if (idbaack) begin
      ioaddr <= (ibaack)? {8'd0,ibadr[24:2],2'd0} : dbadr;
      iosize <= (ibaack)? ibsize : {dbsize,L};
      iside  <= ibaack;
    end

    if (!ioena)      dbcnt <= 0; else dbcnt <= dbcnt+dbinc;
    if (ibaack||rst) ibcnt <= 0; else if (iswr||ioena) ibcnt <= ibcnt+ibinc;

    bdone <= iswr && ((iosize[0] && ibcnt==3) || (iosize[1] && ibcnt==7));
    ioenad <= ioena;

    if (rst) ireq <= 0; else ireq <= (dbaack &&  dbrnw) || (ibaack && !boot) || (ireq && !(iena && !ioenad));
    if (rst) oreq <= 0; else oreq <= (dbaack && !dbrnw) || (oreq && !(oena && !ioenad));
    if (rst) busy <= 0; else busy <= idbaack || (busy && !done);
    if (rst) hold <= 0; else hold <= idbaack || (hold && !ioena && !bdone);

  end

endmodule

module proc_dcr (sclkx, sread_cy, sread_en, swrite_en, saddr,swrbus,
		 dcr_wr, dcr_rd, dcr_ad, dcr_ack, dcr_buso);

  input sclkx, dcr_wr, dcr_rd;
  input [1:0] dcr_ad;
  input [31:0] dcr_buso;
  output [31:0] saddr,swrbus;
  output sread_cy, sread_en, swrite_en, dcr_ack;

  reg[31:0] saddr,swrbus;
  reg dcr_cyc,sread_cy,swrite_en,saddr_en,sdata_en,sread_en;

  always @(posedge sclkx) begin
    dcr_cyc <= (dcr_wr || dcr_rd);
    saddr_en <= dcr_wr &&  dcr_ad[0] && !dcr_cyc;
    sdata_en <= dcr_wr && !dcr_ad[0] && !dcr_cyc;
    sread_en <= dcr_rd && !dcr_ad[0] && !dcr_cyc;
    if (saddr_en) saddr <= dcr_buso;
    if (saddr_en) sread_cy <= dcr_ad[1];
    if (sdata_en) swrbus <= dcr_buso;
    swrite_en <= sdata_en;
  end
  assign dcr_ack = (dcr_wr || dcr_rd);

endmodule

/*
  Xternal processor block
  interface with TI chip
*/
module processorX (
  bclk,baddr, brd,brdbus,bwr,bwrbus, boot,     				// PPC BRam IF
  sclkx,srst, srw,saddr, srdx,srdbus, swrx,swrbus,			// System Control
  sclks,srds,swrs, sclkf,srdf,swrf, sclkt,srdt,swrt,			// Altername SC rates
  ioclk, ireq,iena,ibus, oreq,oena,obus, 				// IO busses
  ioaddr,iosize,iobusy,iok,ioreq,					// IO signals
  test);

  parameter IBW=32;

  input bclk,brd,bwr,boot;
  input [15:0] baddr;
  output [31:0] brdbus;
  input [31:0] bwrbus;

  input srst,sclks,sclkx,sclkf,sclkt;
  input [31:0] srdbus;
  output srw, srds,swrs,srdx,swrx,srdf,swrf,srdt,swrt;
  output [31:0] swrbus,saddr;

  input ioclk,iena,oena,iobusy;
  output ireq,oreq,iok;
  output [31:0] ioaddr;
  output [1:0] iosize;
  output [31:0] ioreq;

  output [IBW-1:0] obus;
  input [IBW-1:0] ibus;

  output [7:0] test;

endmodule



/*
  Processor for IOM or mini-PM
*/
module processorLite (
  cclk,crd,cwr,cpage,caddr,cdata,cdatao, cs,		// JTAG IF
`ifdef PRC_C2SBUS
  c2s,
`endif
  sclkx,srst, srw,saddr, srdx,srdbus, swrx,swrbus, sack,// System Control
`ifndef PRC_NOALTBUS
  sclks,srds,swrs, sclkf,srdf,swrf, 			// Altername SC rates
`endif
  test);

  parameter IBW=32;
  parameter MODE=1;	// mode 0=mainboard 1=processor

  localparam DSBW=32;
  localparam NR=`JVM_NR;
  localparam NCR=`JVM_NCR;

  input cclk,crd,cwr,cs;
  input [7:0] cpage,caddr,cdata;
  output [7:0] cdatao;

  input sclkx,srst;
  input [31:0] srdbus;
  output srw, srdx,swrx;
  output [31:0] swrbus,saddr;
  input [7:0] sack;

`ifndef PRC_NOALTBUS
  input sclks,sclkf;
  output srds,swrs,srdf,swrf;
`endif
`ifdef PRC_C2SBUS
  input c2s;
`endif

  output [7:0] test;

  // Power assignments
  wire H=1, L=0;

  // Debug, status and reset logic
  reg rst,iok,creset,cstart;
  always @(posedge sclkx) begin
    rst <= srst || creset;
    iok <= !rst;
  end

  // OCM - OnChip Memory Controllers 
  wire isen,dsen,dwen;
  wire [21:0] dsadr;

  // Control port interface
  reg [15:0] ccnt;
  reg [7:0] cdatad;
  reg [3:0] cunb;
  reg iswr,dswr,cinc;
  wire cadr = cs && cwr && caddr[2];
  wire csys = cs && cwr && caddr[3];
  always @(posedge cclk) begin
    if (cwr)  cunb <= caddr[3:0];
    if (cadr) ccnt <= {cdata,cdatad};
    else if (cinc) ccnt <= ccnt+1;
    if (cwr) cdatad <= cdata;
    cinc <= cs && ((cwr && caddr[3:2]==0) || crd);
    dswr <= cs &&   cwr && caddr[3:2]==0 && !ccnt[13];
    iswr <= cs &&   cwr && caddr[3:2]==0 &&  ccnt[13];
  end
  always @(posedge cclk or posedge srst) begin
    if (srst) cstart <= 0; else if (csys) cstart <= cdata[0];
    if (srst) creset <= 0; else if (csys) creset <= cdata[1];
  end

  // unused busses
  wire brd,bwr;
  wire [15:0] baddr;
  wire [31:0] brdbus;
  wire [31:0] bwrbus;

  wire ioclk,iena,oena;
  wire ireq,oreq;
  wire [31:0] ioaddr;
  wire [1:0] iosize;
  wire [IBW-1:0] obus;
  wire [IBW-1:0] ibus;

  wire [2:0] dswen;
  wire [IBW-1:0] dsbusi,dsbuso;
  wire dsw = dswen[0] || dswen[1];
  assign bwrbus = dsbuso[31:0];
  dpram #(NR,8,DSBW) dsr0 (cclk,H, ccnt,dswr, cdatad,cdatao, 
                           sclkx,H, dsadr,dsw, dsbuso,dsbusi);
  assign bwr   = dswen[2];
  assign dsadr = baddr[15:2];

  reg cl1,cl2;
  always @(posedge sclkx) begin
    cl1 <= iswr;
    cl2 <= cl1;
  end
  wire cload = cl1 && !cl2;
  wire [11:0] code = {cunb,cdatad};
  wire brd_; assign brd = H; // now enables the DMAC ram

  wire swr_en,srd_en;
  wire [31:0] saddr_,swrbus_;

`ifdef VIRTEX4
  assign srw=0, saddr_=0, swr_en=0, swrbus_=0, srd_en=0;

`else
  jvm #(IBW,NCR,0) j1 (sclkx, rst, cstart,cload,code,
                 sclkx, srw, saddr_,swr_en,swrbus_,srd_en,srdbus,
                 sclkx, baddr,dswen,dsbuso,brd_,dsbusi,brdbus,
                 ioclk, ioaddr,iosize, ireq,iena,ibus, oreq,oena,obus, test);
`endif

  // system busses
  reg srdx,swrx;

`ifdef PRC_C2SBUS
  reg csrw,cswr,csrd;
  reg [31:0] saddr,swrbus;
  always @(posedge sclkx) begin
    cswr <= cwr && c2s;
    csrd <= crd && c2s;
    csrw <= (cswr || csrd);
    saddr <= c2s? {cpage,caddr} : saddr_;
    swrbus <= c2s? cdata : swrbus_;
    if (srst) srdx <= 0; else srdx <= srd_en || (csrd && !csrw);
    if (srst) swrx <= 0; else swrx <= swr_en || (cswr && !csrw);
  end
`else
  always @(posedge sclkx) begin
    if (srst) srdx <= 0; else srdx <= srd_en;
    if (srst) swrx <= 0; else swrx <= swr_en;
  end
  assign saddr = saddr_;
  assign swrbus = swrbus_;
`endif

`ifndef PRC_NOALTBUS

`ifdef PRC_SBUS
  reg swrs,srds,swrs_,srds_,srws;
  always @(posedge sclkx or posedge srws) begin
    if (srws) srds_ <= 0; else srds_ <= srd_en || srds_;
    if (srws) swrs_ <= 0; else swrs_ <= swr_en || swrs_;
  end
  always @(posedge sclks) begin
    srds <= srds_ && !srds;
    swrs <= swrs_ && !swrs;
  end
  always @(negedge sclks) srws <= srds || swrs || srst;
`endif

`ifdef PRC_FBUS
  reg swrf,srdf,swrf_,srdf_,srwf;
  always @(posedge sclkx or posedge srwf) begin
    if (srwf) srdf_ <= 0; else srdf_ <= srd_en || srdf_;
    if (srwf) swrf_ <= 0; else swrf_ <= swr_en || swrf_;
  end
  always @(posedge sclkf) begin
    srdf <= srdf_ && !srdf;
    swrf <= swrf_ && !swrf;
  end
  always @(negedge sclkf) srwf <= srdf || swrf || srst;
`endif

`endif

endmodule

/*
  DSP block for JVM - here for obfuscator
*/

module jvmdsp (clk,ena, sel, stka,stkb, stkl,stko);

  parameter DBW=32;

  input clk,ena;
  input [3:0] sel;		// operation selector
  input [DBW-1:0] stka,stkb;	// stack A/B registers
  input [DBW-1:0] stkl;		// LUT op results
  output [DBW-1:0] stko;	// output register

  wire L=0,H=1;
  wire [6:0] opmode;
  wire [3:0] alumode;
  wire [2:0] aop = sel[2:0];	// 0=tbd 1=and 2=ror 3=xor 4=neg 5=sub 6=add 7=mul
  wire [DBW-1:0] stkp;
  wire isMult = (aop==7);
  wire isNeg  = (aop==4);
  wire isCZ   = (aop==4) || (aop==7);
  wire isROR  = (aop==2);
  assign opmode[1:0] = isMult? 1 : 3;            // 0=0 1=M 2=P 3=A:B
  assign opmode[3:2] = isMult? 1 : isROR? 2 : 0; // 0=0 1=M 2=-1 3=C
  assign opmode[6:4] = isCZ?   0 : 3;            // 0=0 1=PC 2=P 3=C 
  assign alumode = (aop==1)? 12 : (aop==2)? 12 : (aop==3)? 7 : (aop==4)? 3 : (aop==5)? 3 : 0;

  wire [24:0] stkba = isMult? stkb : stka[31:18]; // put stka in portB and stkb in portA for subtract needs

`ifndef verilator
`ifdef SPARTAN6
// warning - this DSP has no ALU for the bit compare functions
  DSP48A1 dsp (.CLK(clk),.OPMODE(opmode), .A(stkba),.B(stka),.C(stkb),.P(stkp),
		.CEP(ena), .CEA(L),.CEB(L),.CEC(L),.CED(L),.CECARRYIN(L),.CEOPMODE(L));
  defparam dsp.A0REG=0;
  defparam dsp.B0REG=0;
  defparam dsp.CREG=0, dsp.MREG=0, dsp.PREG=1;
//  defparam dsp.OPMODEREG=0, dsp.ALUMODEREG=0;
`else
  DSP48E dsp (.CLK(clk),.OPMODE(opmode),.ALUMODE(alumode), .A(stkba),.B(stka),.C(stkb),.P(stkp),
		.CEP(ena), .CEC(L),.CEM(L),.CEA1(L),.CEA2(L),.CEB1(L),.CEB2(L));
  defparam dsp.AREG=0, dsp.ACASCREG=0;
  defparam dsp.BREG=0, dsp.BCASCREG=0;
  defparam dsp.CREG=0, dsp.MREG=0, dsp.PREG=1;
  defparam dsp.OPMODEREG=0, dsp.ALUMODEREG=0;
`endif
`ifdef VIRTEX5
  defparam dsp.USE_MULT="MULT";
`endif
`endif

  reg stksel; always @(posedge clk) if (ena) stksel <= sel[3];
  assign stko = stksel? stkp : stkl;

endmodule
  
