/**********************************************
 ICE Proprietary Software - do NOT disseminate
 **********************************************/
/*
  Test port handler

  Jeff Schoen
  Innovative Computer Engineering, Inc.
  1/15/2003

  The MPCIO bus consists of 1 to 4 two wire interfaces often I2C
  The MPCIO byte[5:4] are the interface index and byte[1:0] are the values and byte[3:2] the driver enables

*/
`ifdef PIC
`define TP_HAS_J4
`define TP_HAS_STATUS
`endif
`ifdef PIC5
`define TP_HAS_MPIO
`endif
`ifdef PIC6
`define TP_HAS_MPIO
`endif

module testport (
  clk,rst, rd,wr,cs,addr,data,datao,
  tp,tpi,
  oppms,sig,
`ifdef TP_HAS_J4
  pj4, 
`endif
`ifdef TP_HAS_MPIO
  mpcio,
`endif
  tp0,tp1,tp2,tp3,tp4,tp5,tp6,tp7);

  parameter TP_OE=0;
  parameter TP_SEL=0;

  input clk,rst,cs,rd,wr;
  input [7:0] addr,data;
  output [7:0] datao;
  inout [7:0] tp;
  output [8:0] tpi;
  input oppms;
  input [3:0] sig;
`ifdef TP_HAS_J4
  input pj4;
`else
  wire pj4=0;
`endif
`ifdef TP_HAS_MPIO
  inout [7:0] mpcio;
`else
  wire [7:0] mpcio;
`endif
  input [7:0] tp0,tp1,tp2,tp3,tp4,tp5,tp6,tp7;

  wire L=0,H=1;
  genvar i;

`ifdef PIC6
  assign pod = sig[0];
`else
  assign pod = L;
`endif

  reg tpoe,tpts;
  reg [3:0] tpsel;
  reg [7:0] status,mpreg;
  wire [1:0] ad = addr[1:0];
  wire wtps = cs&(ad==0)&wr;
  wire wtpd = cs&(ad==1)&wr;
  wire rtps = cs&(ad==1)&rd;
  wire wtpm = cs&(ad==3)&wr;
  wire rstn; delaypipe #(15) dl1 (clk,!rst,rstn);
  wire rstd = !rstn;
  //always @(posedge clk or posedge rstd) begin
  always @(posedge clk) begin
    if (rstd) tpoe  <= TP_OE;  else if (wtps) tpoe  <= data[4];
    if (rstd) tpsel <= TP_SEL; else if (wtps) tpsel <= data[3:0];
    if (rstd) tpts  <= !TP_OE && !pod;  else if (wtps) tpts  <= !data[4] && !pod;
  end

  // generate the test status array for PICs
`ifdef TP_HAS_STATUS 
  reg [9:0] scount;
  wire silon = !tpsel[3];
  always @(posedge clk) if (oppms) scount <= scount+1;
  generate
  for (i=0; i<8; i=i+1) begin:blk
    always @(posedge clk) if (wtpd) status[i] <= data[i]; else if (silon) status[i] <= (scount[9:6]==i || scount[9:6]==(15-i));
  end
  endgenerate
`endif
`ifdef TP_HAS_MPIO
  wire [15:0] mpdef = pj4? 16'h00C0 : 16'h0000;
  generate
  for (i=0; i<4; i=i+1) begin:mpr
    reg [3:0] mpreg; 
    always @(posedge clk or posedge rstd) begin
      if (rstd) mpreg <= mpdef[i*4+3:i*4];  else if (wtpm && data[6:5]==i) mpreg <= data[3:0];
    end
    assign mpcio[i*2+1:i*2] = {mpreg[3]?mpreg[1]:1'bZ,mpreg[2]?mpreg[0]:1'bZ};
  end
  endgenerate
`else
  assign mpcio = { H,H,H,H, sig };
`endif

  // mux test sources
`ifdef VIVADO
  (* dont_touch = "yes" *) wire [7:0] tpo;
`else
  wire [7:0] tpo;
`endif
`ifdef NOTESTMUXX
  assign tpo = tp0;
`else
  mux8xN #(8) tpm (tpo, tp0,tp1,tp2,tp3,tp4,tp5,tp6,tp7, tpsel[2:0]);
`endif

`ifdef TESTMUXC
  reg [7:0] tpc;
  generate
  for (i=0; i<8; i=i+1) begin:bit
    always @(posedge tpo[i] or posedge rtps) if (rtps) tpc[i] <= 0; else tpc[i] <= 1;
  end
  endgenerate
`else
  wire [7:0] tpc = 0;
`endif

  // assign the signature and debug ROM 
`ifdef BPROM
  wire crd  = cs&rd;
  wire crst = cs&rd&addr[2];
  wire [7:0] romo,teste;
  bprom_access bpa (clk,crst,crd,romo,teste);
  mux4xN #(8) tdm (datao, tpo,tpc,status,romo, addr[5:4]);
`else
  wire [31:0] idat = {mpcio,status,tpc,tpo};
 `ifdef VIVADO
//  this 0'h0 is some wierd quirk of vivado 2015.4 - tried every other combo known to man
  RAMB16_S9_S36 ram (.CLKA(clk),.ENA(H),.WEA(L),.SSRA(L),.ADDRA({0'h0,addr[6:0]}),.DIA(8'b0),.DIPA(1'b0),.DOA(datao),
                     .CLKB(clk),.ENB(H),.WEB(H),.SSRB(L),.ADDRB(8'h02),  .DIB(idat),.DIPB(4'b0));
`else
  RAMB16_S9_S36 ram (.CLKA(clk),.ENA(H),.WEA(L),.SSRA(L),.ADDRA(addr[6:0]),.DIA(8'b0),.DIPA(1'b0),.DOA(datao),
                     .CLKB(clk),.ENB(H),.WEB(H),.SSRB(L),.ADDRB(8'h02),  .DIB(idat),.DIPB(4'b0));
`endif
`ifdef HSV
  defparam ram.INIT_00 = `HSV;
`endif
`endif

`ifdef PIC8P
  assign tp = tpo;
`else
  iobuftN #(8) tpb (tp,tpts,tpi[7:0],tpoe?tpo:status);
`endif
  assign tpi[8] = tpoe;

endmodule


module bprom_access (clk,rst,crd,odat,test);
  input clk,rst,crd;
  output [7:0] odat,test;

  wire L=0, H=1;
  wire eos,val;
  wire [31:0] idat;
  reg [11:0] oadr;
  reg [9:0] iadr;
  reg [3:0] cnt;
  reg load,last,udone;

  always @(posedge clk) begin
    cnt <= cnt+1;
    if (!eos) iadr <= 0; else if (load) iadr <= iadr+1;
    if (!eos) udone <= 0; else if (last) udone <= H;
    if (rst) oadr <= 0; else if (crd) oadr <= oadr+1;
    load <= val && (cnt[1:0]==0);
    last <= load && (iadr==9'h1FF);
  end
  wire iclk = cnt[1];

`ifdef VIRTEX4
  STARTUP_VIRTEX4 sv4 (.EOS(eos),.USRCCLKO(iclk),.USRCCLKTS(udone),.USRDONEO(udone),.USRDONETS(L));
  USR_ACCESS_VIRTEX4 uav4 (.DATA(idat),.DATAVALID(val));
  RAMB16_S9_S36 ram (.CLKB(clk),.ENB(H),.WEB(load),.ADDRB(iadr),.DIB(idat), .CLKA(clk),.ENA(H),.ADDRA(oadr),.DIA(8'b0),.DOA(odat));
  defparam ram.INIT_00 = {256{1'b1}};
  defparam ram.INIT_01 = {256{1'b1}};
  defparam ram.INIT_02 = {256{1'b1}};
  defparam ram.INIT_03 = {256{1'b1}};
  defparam ram.INIT_04 = {256{1'b1}};
  defparam ram.INIT_05 = {256{1'b1}};
  defparam ram.INIT_06 = {256{1'b1}};
  defparam ram.INIT_07 = {256{1'b1}};
  defparam ram.INIT_08 = {256{1'b1}};
  defparam ram.INIT_09 = {256{1'b1}};
  defparam ram.INIT_0A = {256{1'b1}};
  defparam ram.INIT_0B = {256{1'b1}};
  defparam ram.INIT_0C = {256{1'b1}};
  defparam ram.INIT_0D = {256{1'b1}};
  defparam ram.INIT_0E = {256{1'b1}};
  defparam ram.INIT_0F = {256{1'b1}};
  defparam ram.INIT_10 = {256{1'b1}};
  defparam ram.INIT_11 = {256{1'b1}};
  defparam ram.INIT_12 = {256{1'b1}};
  defparam ram.INIT_13 = {256{1'b1}};
  defparam ram.INIT_14 = {256{1'b1}};
  defparam ram.INIT_15 = {256{1'b1}};
  defparam ram.INIT_16 = {256{1'b1}};
  defparam ram.INIT_17 = {256{1'b1}};
  defparam ram.INIT_18 = {256{1'b1}};
  defparam ram.INIT_19 = {256{1'b1}};
  defparam ram.INIT_1A = {256{1'b1}};
  defparam ram.INIT_1B = {256{1'b1}};
  defparam ram.INIT_1C = {256{1'b1}};
  defparam ram.INIT_1D = {256{1'b1}};
  defparam ram.INIT_1E = {256{1'b1}};
  defparam ram.INIT_1F = {256{1'b1}};
  defparam ram.INIT_20 = {256{1'b1}};
  defparam ram.INIT_21 = {256{1'b1}};
  defparam ram.INIT_22 = {256{1'b1}};
  defparam ram.INIT_23 = {256{1'b1}};
  defparam ram.INIT_24 = {256{1'b1}};
  defparam ram.INIT_25 = {256{1'b1}};
  defparam ram.INIT_26 = {256{1'b1}};
  defparam ram.INIT_27 = {256{1'b1}};
  defparam ram.INIT_28 = {256{1'b1}};
  defparam ram.INIT_29 = {256{1'b1}};
  defparam ram.INIT_2A = {256{1'b1}};
  defparam ram.INIT_2B = {256{1'b1}};
  defparam ram.INIT_2C = {256{1'b1}};
  defparam ram.INIT_2D = {256{1'b1}};
  defparam ram.INIT_2E = {256{1'b1}};
  defparam ram.INIT_2F = {256{1'b1}};
  defparam ram.INIT_30 = {256{1'b1}};
  defparam ram.INIT_31 = {256{1'b1}};
  defparam ram.INIT_32 = {256{1'b1}};
  defparam ram.INIT_33 = {256{1'b1}};
  defparam ram.INIT_34 = {256{1'b1}};
  defparam ram.INIT_35 = {256{1'b1}};
  defparam ram.INIT_36 = {256{1'b1}};
  defparam ram.INIT_37 = {256{1'b1}};
  defparam ram.INIT_38 = {256{1'b1}};
  defparam ram.INIT_39 = {256{1'b1}};
  defparam ram.INIT_3A = {256{1'b1}};
  defparam ram.INIT_3B = {256{1'b1}};
  defparam ram.INIT_3C = {256{1'b1}};
  defparam ram.INIT_3D = {256{1'b1}};
  defparam ram.INIT_3E = {256{1'b1}};
  defparam ram.INIT_3F = {256{1'b1}};
`endif
  assign test = {load,crd,!udone,iclk, eos,clk,val,rst};

endmodule
