# STEP#1: define the output directory area.
#
set part [lindex $argv 0]
set desn [lindex $argv 1]
set card [lindex $argv 2]
set io   [lindex $argv 3]
set pe   [lindex $argv 4]
set hsv  [lindex $argv 5]
puts "vFlow part=$part design=$desn card=$card io=$io p2=$pe hsv=$hsv"
create_project -in_memory -part $part
#
# STEP#2: setup design sources and constraints (read the top.prj file)
# 
set f [open top.prj]
while {[gets $f line] > -1} {
  if { [string first # $line] > -1} continue
  set fn [string range $line 14 end-1]
        if { [string last .v $fn] > -1}    { read_verilog [ glob $fn ]
  } elseif { [string last .sv $fn] > -1}   { read_verilog -sv [ glob $fn ]
  } elseif { [string last .vhdl $fn] > -1} { read_vhdl [ glob $fn ] 
  } elseif { [string last .edif $fn] > -1} { read_edif [ glob $fn ]
  } elseif { [string last .dcp $fn] > -1}  { read_checkpoint [ glob $fn ]
  } elseif { [string last .xci $fn] > -1}  { read_ip [ glob $fn ]
  } elseif { [string last .xdc $fn] > -1}  { read_xdc [ glob $fn ]
  } elseif { [string last .tcl $fn] > -1}  { source [ glob $fn ] 
  } else                                   { add_files [ glob $fn ] 
  }
}
close $f
read_xdc top.xdc
##if { [file exists pre_synth.tcl] } { source pre_synth.tcl }
#
# STEP#3: run synthesis, write design checkpoint, report timing, and utilization estimates
#
synth_design -top top -part $part -include_dirs ../lib -verilog_define VIVADO=true
write_checkpoint -force post_synth.dcp
report_timing_summary -file post_synth_timing_summary.rpt
report_utilization -file post_synth_util.rpt
#
# STEP#4: run logic optimization, placement and physical logic optimization, write design checkpoint, report utilization and timing estimates
#
opt_design
write_checkpoint -force -noxdef post_opt.dcp
report_drc -file post_opt_drc.rpt
#
# STEP#4+: run placer
#
place_design
write_checkpoint -force post_place.dcp
report_clock_utilization -file clock_util.rpt
report_utilization -file post_place_util.rpt
report_timing_summary -file post_place_timing_summary.rpt
report_io -file post_place_io.rpt
#
# STEP#5: run the router, write the post-route design checkpoint, report routing status, timing, power, and DRC, and finally save the Verilog netlist.
#
route_design 
write_checkpoint -force post_route.dcp
report_route_status -file post_route_status.rpt
report_timing_summary -file post_route_timing_summary.rpt
report_power -file post_route_power.rpt
report_drc -file post_imp_drc.rpt
write_verilog -force cpu_impl_netlist.v -mode timesim -sdf_anno true
#
write_xdc -force top.xdc.out
# phys_opt_design
# write_checkpoint -force post_route.dcp
# report_timing_summary -file -check_timing_verbose post_route_timing_summary.rpt
#
# STEP#6: generate a bitstream (Valid Config Rates K8 = 3,6,9,12,22,33,40,50,69,90)
# STEP#6: generate a bitstream (Valid Config Rates K7 = 3,6,9,12,22,33,40,50,66)
#
if { [string first xcku $part] > -1} {
set_property BITSTREAM.Config.ConfigRate 90 [current_design]
} elseif { [string first xcvu $part] > -1} {
set_property BITSTREAM.Config.ConfigRate 85.0 [current_design]
} else {
set_property BITSTREAM.Config.ConfigRate 66 [current_design]
}
set_property BITSTREAM.Config.UserID 0x0001CE1D [current_design]
set_property BITSTREAM.Config.SPI_BusWidth 4 [current_design]
set_property BITSTREAM.Config.SPI_Fall_Edge Yes [current_design]
write_bitstream -force top.bit
exit
